﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Optimization.Operators {
  /// <summary>
  /// A base class for operators that perform path relinking.
  /// </summary>
  [Item("PathRelinker", "A base class for operators that perform path relinking.")]
  [StorableClass]
  public abstract class PathRelinker : SingleSuccessorOperator, IPathRelinker {
    #region Parameter properties
    public ScopeParameter CurrentScopeParameter {
      get { return (ScopeParameter)Parameters["CurrentScope"]; }
    }
    public ILookupParameter<ItemArray<IItem>> ParentsParameter {
      get { return (IScopeTreeLookupParameter<IItem>)Parameters["Parents"]; }
    }
    public IValueParameter<PercentValue> RelinkingAccuracyParameter {
      get { return (IValueParameter<PercentValue>)Parameters["RelinkingAccuracy"]; }
    }
    #endregion

    #region Properties
    private IScope CurrentScope {
      get { return CurrentScopeParameter.ActualValue; }
    }
    private ItemArray<IItem> Parents {
      get { return ParentsParameter.ActualValue; }
    }
    private PercentValue RelinkingAccuracy {
      get { return RelinkingAccuracyParameter.Value; }
    }
    #endregion

    [StorableConstructor]
    protected PathRelinker(bool deserializing) : base(deserializing) { }
    protected PathRelinker(PathRelinker original, Cloner cloner) : base(original, cloner) { }
    protected PathRelinker()
      : base() {
      #region Create parameters
      Parameters.Add(new ScopeParameter("CurrentScope"));
      Parameters.Add(new ScopeTreeLookupParameter<IItem>("Parents"));
      Parameters.Add(new ValueParameter<PercentValue>("RelinkingAccuracy", new PercentValue(0.25)));
      #endregion
    }

    public sealed override IOperation Apply() {
      ItemArray<IItem> relinkedSolutions = Relink(Parents, RelinkingAccuracy);
      for (int i = 0; i < relinkedSolutions.Length; i++)
        CurrentScope.Variables.Add(new Variable(string.Format("Child {0}", i), relinkedSolutions[i]));
      return base.Apply();
    }

    protected abstract ItemArray<IItem> Relink(ItemArray<IItem> parents, PercentValue n);
  }
}